#include "renderer2d.h"
#include "renderer.h"
#include "shader.h"
#include "skip.h"

#include <stdlib.h>

#include "gl.h"

struct vert {
	struct sw_vec2f pos;
	struct sw_vec4f color;
};

static void r_enter(struct sw_window *win, void *data);

static void r_exit(struct sw_window *win, void *data);

struct sw_renderer2d *sw_renderer2d_create() {
	struct sw_renderer2d *ren;
	char *frag, *vert;

	ren = malloc(sizeof(struct sw_renderer2d));
	ren->renderer = sw_renderer_create(ren, r_enter, r_exit);

	frag = sw_skip_get_string(sw_skip_self, "renderer2d_frag.glsl");
	vert = sw_skip_get_string(sw_skip_self, "renderer2d_vert.glsl");

	ren->shader = sw_shaderprogram_create((u8 *) vert, 0, (u8 *) frag, 0, 0);
	ren->vertex_buffer = sw_vertex_buffer_create(sizeof(struct vert));
	ren->color = sw_vec4f(1.f, 1.f, 1.f, 1.f);

	glGenVertexArrays(1, &ren->vao);
	glBindVertexArray(ren->vao);

	glGenBuffers(1, &ren->vbo);
	glBindBuffer(GL_ARRAY_BUFFER, ren->vbo);

	/* todo: vertex attrib thingies */
	glVertexAttribPointer(
		0, 2, GL_FLOAT, GL_FALSE, sizeof(struct vert), 0
	);
	glEnableVertexAttribArray(0);

	glBindVertexArray(0);

	return ren;
}

void sw_renderer2d_draw_quad(
	struct sw_renderer2d *ren,
	struct sw_vec2f a,
	struct sw_vec2f b,
	struct sw_vec2f c,
	struct sw_vec2f d
) {
	struct vert vert;

	vert.color = ren->color;

	vert.pos = a;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);

	vert.pos = b;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);

	vert.pos = c;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);

	vert.pos = b;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);

	vert.pos = d;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);

	vert.pos = c;
	sw_vertex_buffer_add(ren->vertex_buffer, &vert);
}

void sw_renderer2d_set_color(struct sw_renderer2d *ren, struct sw_vec4f color) {
	ren->color = color;
}

static void r_enter(struct sw_window *win, void *data) {
	struct sw_renderer2d *ren;

	(void) win;

	ren = data;
	sw_vertex_buffer_clear(ren->vertex_buffer);

	sw_renderer2d_draw_quad(
		ren,
		sw_vec2f(-0.5f, -0.5f),
		sw_vec2f(0.5f, -0.5f),
		sw_vec2f(-0.5f, 0.5f),
		sw_vec2f(0.5f, 0.5f));

	glBindVertexArray(ren->vao);
	sw_shaderprogram_use(ren->shader);
	sw_vertex_buffer_send(ren->vertex_buffer);
	glDrawArrays(GL_TRIANGLES, 0, (i32) ren->vertex_buffer->elem_count);
	glBindVertexArray(0);
}

static void r_exit(struct sw_window *win, void *data) {
	(void) win;
	(void) data;

}
